local L = LibStub("AceLocale-3.0"):GetLocale("TrackOMatic", true);

local function owlcatStoneProgress(state, label)
    if (state) then
        return "|cff00ff00" .. label .. "|r";
    else
        return "|cff808080" .. label .. "|r";
    end
end

--========================================
-- Update an event/trigger bar
--========================================
function TrackOMatic_SetEventTriggerBar(data)

    local value, max, color, label, levelLabel, tooltipTitle, tooltipText, menuTitle, clickFunc;

    tooltipTitle = "";
    tooltipText = "";

    local menu = {};
    local isGlowing = false;
    local glowColor = {r = 1, g = 1, b = 0};

    -- ==================== Burning Plate of the Worldbreaker ==================== --
    if (data.id == "0001") then
        -- set default label and tooltip title
        label = "Burning Plate of the Worldbreaker";
        tooltipTitle = "Burning Plate of the Worldbreaker";
        tooltipText = "Hidden Artifact Appearance - Protection Warrior\n\n";

        local bpAvailable = IsQuestFlaggedCompleted(44311);
        local bpDenied = IsQuestFlaggedCompleted(44312);
        
        max = 1;
        value = 1;
        if (bpDenied == true) then
            color = {r = 1, g = 0, b = 0};
            tooltipText = tooltipText .. "Try again tomorrow.";
        elseif (bpAvailable == true) then
            color = {r = 0, g = 1, b = 0};
            tooltipText = tooltipText .. "Shield is available!";
            isGlowing = true;
        else
            value = 0;
            color = {r = 0.5, g = 0.5, b = 0.5};
            tooltipText = tooltipText .. "Head to Neltharion's Vault";
        end
        levelLabel = "";

        menuTitle = "Burning Plate of the Worldbreaker";

    -- ==================== Feather of the Moonspirit ==================== --
    elseif (data.id == "0002") then
        -- set default label and tooltip title
        label = "Feather of the Moonspirit";
        tooltipTitle = "Feather of the Moonspirit";
        tooltipText = "Hidden Artifact Appearance - Feral Druid\n\n";

        local dreamwayRoll = IsQuestFlaggedCompleted(44326);
        local feralasStatueUp = IsQuestFlaggedCompleted(44327);
        local hinterlandsStatueUp = IsQuestFlaggedCompleted(44328);
        local duskwoodStatueUp = IsQuestFlaggedCompleted(44329);
        local duskwoodStatueTouched = IsQuestFlaggedCompleted(44330);
        local feralasStatueTouched = IsQuestFlaggedCompleted(44331);
        local hinterlandsStatueTouched = IsQuestFlaggedCompleted(44332);
        
        max = 1;
        value = 0;
        if (dreamwayRoll == true) then
            if (feralasStatueUp or hinterlandsStatueUp or duskwoodStatueUp) then
                value = 0;
                max = 3;
                isGlowing = false;
                color = {r = 0, g = 1, b = 0};
                tooltipText = tooltipText .. owlcatStoneProgress(duskwoodStatueTouched, "Duskwood stone touched") .. "\n";
                tooltipText = tooltipText .. owlcatStoneProgress(feralasStatueTouched, "Feralas stone touched") .. "\n";
                tooltipText = tooltipText .. owlcatStoneProgress(hinterlandsStatueTouched, "Hinterlands stone touched") .. "\n";
                if (feralasStatueTouched) then
                    value = value + 1;
                elseif (feralasStatueUp) then
                    isGlowing = true;
                    tooltipText = tooltipText .. "\nGo to Feralas!";
                end
                if (hinterlandsStatueTouched) then
                    value = value + 1;
                elseif (hinterlandsStatueUp) then
                    isGlowing = true;
                    tooltipText = tooltipText .. "\nGo to Hinterlands!";
                end
                if (duskwoodStatueTouched) then
                    value = value + 1;
                elseif (duskwoodStatueUp) then
                    isGlowing = true;
                    tooltipText = tooltipText .. "\nGo to Duskwood!";
                end
            else
                value = 1;
                color = {r = 1, g = 0, b = 0};
                tooltipText = tooltipText .. "Try again tomorrow.";
            end
        else
            value = 0;
            color = {r = 0.5, g = 0.5, b = 0.5};
            tooltipText = tooltipText .. "Teleport to the Dreamway";
        end
        levelLabel = "";

        menuTitle = "Feather of the Moonspirit";

    else
        max = 1;
        value = 1;
        color = {r = 0.5, g = 0.5, b = 0.5};
        label = L["UNDEFINED"];
    end

    return value, max, color, label, levelLabel, tooltipTitle, tooltipText, menuTitle, menu, clickFunc, isGlowing, glowColor;
end

--========================================
-- Add an entry to the event/trigger
-- category
--========================================
function TrackOMatic_AddEventTrigger(id)
    local eventTriggers = {
        ["0001"] = 1,       -- Burning Plate of the Worldbreaker
        ["0002"] = 1,       -- Feather of the Moonspirit
    };
    if (not eventTriggers[id]) then
        return;
    end
    if (TrackOMatic_AddToCategory("eventtrigger", {id = id})) then
        TrackOMatic_Message(string.format(L["TRACKER_ADD_EVENTTRIGGER"], TrackOMatic_GetEventTriggerLabel(id)));
    end
end

--========================================
-- Remove an event/trigger bar
--========================================
function TrackOMatic_RemoveEventTrigger(index, isSilent)
    local data = TrackOMatic_RemoveFromCategory("eventtrigger", index);
    if ((not isSilent) and data) then
        local name;
        name = TrackOMatic_GetEventTriggerLabel(data.id);
        if (name) then
            TrackOMatic_Message(string.format(L["TRACKER_REMOVE_EVENTTRIGGER"], name));
        end
    end
end

--========================================
-- Return a localized label for the
-- specified event/trigger
--========================================
function TrackOMatic_GetEventTriggerLabel(id)

    if (string.lower(id) == "0001") then
        return L["EVENT_TRIGGER_0001"];
    end
    
    -- space for rent
    
    return L["UNDEFINED"];

end

function TrackOMatic_SuppressEventTriggerAlert(data)
    if (data.id == "0001") then
    end
end
